/*-
 * Copyright (c) 1999, 2000, 2001, 2002, 2003 Lev Walkin <vlm@lionet.info>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sf_fpos.h,v 1.2 2005/05/25 21:55:10 vlm Exp $
 */

#ifndef __SF_LPOS_H__
#define __SF_LPOS_H__

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif	/* __cplusplus */


/*******************************/
/*** File position functions ***/
/*******************************/

/*
 * initfdpos() creates a position indicator file "filename.ext", and
 * returns its file descriptor. If the position file is already exists,
 * then initfdpos() advances stream's position according to the information
 * derived from that file.
 * 
 * RETURN VALUES:
 * 
 * Position file descriptor, or -1 in case of failure.
 */
int initfdpos(const char *filename, int stream, const char *ext);
int initFILEpos(const char *filename, FILE *stream, const char *ext);

/*
 * adjfdpos() writes the current position within the file referenced by stream
 * into the posfd. If doSync is not zero, it also does fsync(posfd) to submit
 * changes to disk.
 * 
 * RETURN VALUES:
 * 
 * Return values: 0 (Success) or -1 in case of any errors.
 */
int adjfdpos(int stream, int posfd, int doSync);
int adjFILEpos(FILE *stream, int posfd, int doSync);


#ifdef __cplusplus
}
#endif	/* __cplusplus */

#endif	/* __SF_LPOS_H__ */
